// ___SETA43  _____
// 27/11/2024 _____
//RELOJ DS1307
//Arduino 1.8.19
// seta43.duckdns.org
// seta43.blogspot.com

#include "Rotary.h";

#include "RTClib.h"

#include <Adafruit_GFX.h>
#include <Adafruit_SSD1306.h>
/////////////////////////////////////////////////////////////////

#define ROTARY_PIN1	2
#define ROTARY_PIN2	3
#define BUTTON_PIN	4
#define CLICKS_PER_STEP   4   // this number depends on your rotary encoder 

int modo = 0; 
/////////////////////////////////////////////////////////////////
Rotary r;
RTC_DS1307 rtc;
/////////////////////////////////////////////////////////////////

int segundo,minuto,hora,dia,mes;
long anio; //variable año
DateTime HoraFecha;

char Nombres[8][7]= {"*","DIA  ", "MES  ", "YEAR ", "HORA ", "MIN. ",  "SEG. "};

// On an arduino NANO:       A4(SDA), A5(SCL)
#define OLED_RESET     -1 // Reset pin # (or -1 if sharing Arduino reset pin)
#define SCREEN_ADDRESS 0x3C //128x64, 0x3C 
Adafruit_SSD1306 display(128, 64, &Wire, OLED_RESET);

int segundoOld;
char cadena[40];

void setup() 
{
  Serial.begin(9600);
  while (!Serial) {;  }
  Serial.println("\n\nSimple RELOJ");
  
  r.begin(ROTARY_PIN1, ROTARY_PIN2, CLICKS_PER_STEP);
  r.setLeftRotationHandler(showDirection);
  r.setRightRotationHandler(showDirection);

  pinMode(BUTTON_PIN, INPUT_PULLUP);  

  rtc.begin(); //Inicializamos el RTC

  if(!display.begin(SSD1306_SWITCHCAPVCC, SCREEN_ADDRESS)) 
        {
          Serial.println(F("SSD1306 allocation failed"));
          for(;;); // Don't proceed, loop forever
        }

  display.display();
  delay(200);
}



void loop()
{
    delay(1);
    r.loop();

    if(modo==0)
        {
            HoraFecha = rtc.now(); //obtenemos la hora y fecha actual 
            segundo=HoraFecha.second();
            if(segundo!=segundoOld)
              {    
                segundoOld=segundo;
                minuto=HoraFecha.minute();
                hora=HoraFecha.hour();
                dia=HoraFecha.day();
                mes=HoraFecha.month();
                anio=HoraFecha.year();
                printLinea();
                printPanel();
               }
        }
   
    if (digitalRead(BUTTON_PIN) == 0) 
        {     
          delay(100);
          while(digitalRead(BUTTON_PIN) == 0) {;}
          modo = modo + 1;
          if (modo > 6) 
              { 
                modo = 0;
                rtc.adjust(DateTime(anio, mes, dia, hora, minuto, segundo));
              }
          printLinea(); 
          printPanel();
        }
}


////////////RUTINAS/////////////

void printLinea(void)
{
  if(modo!=0) 
  {
    sprintf(cadena,"CAMBIO %s",Nombres[modo]);
    Serial.print(cadena);
  }
  sprintf(cadena,"%02d:%02d:%02d   %02d/%02d/%02d",hora,minuto,segundo,dia,mes,anio);
  Serial.println(cadena);
}


void printPanel(void)
{
  display.clearDisplay(); 

  if(modo!=0) 
  {   
    display.setTextSize(1);
    display.setCursor(0,52);
    sprintf(cadena,"CAMBIO %s",Nombres[modo]);
    display.print(cadena);


    switch(modo)
        {
          case 1:
               display.setCursor(0,37); 
          break;
          case 2:
              display.setCursor(36,37); 
          break;
          case 3:
              display.setCursor(96,37); 
          break;
          case 4:       
             display.setCursor(12,9); 
          break;
          case 5:       
             display.setCursor(48,9); 
          break;
          case 6:
            display.setCursor(84,9);        
          break;
        }
    display.setTextSize(2);  //12x
    display.print("--");
    
  }
   
  display.setTextSize(2);
  display.setTextColor(SSD1306_WHITE);
  
  sprintf(cadena," %02d:%02d:%02d",hora,minuto,segundo);
  display.setCursor(0,0); 
  display.print(cadena);

  sprintf(cadena,"%02d/%02d/%02d",dia,mes,anio);
  display.setCursor(0,28);
  display.print(cadena);

  display.display();
}


void showDirection(Rotary& r) 
{
 int z;
  if(r.getDirection()==1) z=-1; else z=1;

  switch(modo)
    {
      case 1:
          dia+=z;
      break;
      case 2:
          mes+=z;
      break;
      case 3:
          anio+=z;
      break;
      case 4:       
         hora+=z;
      break;
      case 5:       
         minuto+=z;
      break;
      case 6:
        segundo+=z;       
      break;
    }
  printLinea();
  printPanel(); 
}
